@@ -75,7 +75,7 @@ nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 nanotest(identical(n$recv("character", block = 500), c("test", "", "spec")))
 nanotestz(n1$send(c(1L, 0L, 0L), mode = "r"))
 nanotest(is.integer(n$recv("int")))
-nanotestaio(saio <- n$send_aio(data.frame(random(1e4L)), timeout = 900))
+nanotestaio(saio <- n$send_aio(data.frame(random(1e3L), mode = 3L), timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
 nanotest(is.integer(saio$result))
@@ -156,6 +156,8 @@ nanotestz(close(n$listener[[1]]))
 nanotestw(close(n$listener[[1]]) == 12L)
 nanotestz(close(n1$dialer[[1]]))
 nanotestw(close(n1$dialer[[1]]) == 12L)
+nanotestz(reap(n$listener[[2]]))
+nanotestz(reap(n$dialer[[2]]))
 nanotestz(n$close())
 nanotestz(n1$close())
 nanotestw(n1$close() == 7L)
@@ -213,8 +215,8 @@ nanotest(is.integer(req$send(as.raw(1L), mode = 2L, block = 500)))
 nanotestaio(msg <- call_aio(recv_aio(ctx, mode = 8L, timeout = 500)))
 nanotestaio(envraw <- send_aio(ctx, msg, mode = 2L, timeout = 500))
 nanotest(identical(req$recv(mode = 8L, block = 500), as.raw(1L)))
-nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500))
-nanotest(is.integer(reply(ctx, execute = identity, send_mode = "ra", timeout = 500)))
+nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
+nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
 nanotest(inherits(cv <- cv(), "conditionVariable"))
@@ -223,14 +225,25 @@ nanotest(until(cv, 10L))
 nanotest(until(cv, 10))
 nanotestz(cv_reset(cv))
 nanotestz(cv_value(cv))
-nanotestaio(cr <- request_signal(req$context, "test", timeout = 500, cv = cv))
-nanotestaio(cr <- recv_aio_signal(rep, timeout = 500, cv = cv))
+nanotestaio(cs <- request_signal(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
+nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
+nanotestaio(cr <- recv_aio_signal(ctxn, cv = cv, timeout = 500))
 nanotest(call_aio(cr)$data == "test")
-nanotestaio(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
+nanotest(is.integer(send(ctxn, TRUE, mode = 0L, block = FALSE)))
+nanotest(is.integer(recv(ctxn, mode = 8L, block = FALSE)))
+nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
+nanotestaio(cs <- request(.context(req$socket), data = TRUE, send_mode = "raw", ack = TRUE))
+nanotest(recv(ctxn, mode = "logical", block = 500))
+nanotestz(send(ctxn, TRUE, mode = 0L, block = 500))
+nanotestz(cv_reset(cv))
+nanotestaio(cr <- recv_aio_signal(rep, cv = cv, mode = 8L))
+nanotest(wait(cv))
+nanotestz(reap(ctxn))
+nanotestz(length(.subset2(cr, "data")))
 nanotestz(pipe_notify(rep, cv))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
-nanotesterr(recv_aio_signal(err, timeout = 500, cv = cv))
+nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
 nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
 nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
 nanotesterr(wait(err), "valid")
@@ -305,8 +318,9 @@ nanotestnano(bus)
 nanotestw(listen(bus, url = "test") == 3L)
 nanotestw(dial(bus, url = "test") == 3L)
 nanotestz(close(bus))
+nanotestw(close(bus) == 7L)
 nanotestz(close(push))
-nanotestz(close(pull))
+nanotestz(reap(pull))
 nanotestz(survey$close())
 nanotestz(respond$close())
 nanotesterr(socket(protocol = "newprotocol"), "protocol")
@@ -363,11 +377,12 @@ nanotestn(msleep(1))
 nanotestn(msleep("a"))
 nanotest(is.character(urlp <- parse_url("://")) && length(urlp) == 10L)
 nanotest(all(nzchar(parse_url("wss://use:r@[::1]/path?q=1#name"))))
-nanotest(is.double(random()))
-nanotest(length(random(2)) == 2L)
-nanotest(length(random(3L)) == 3L)
-nanotesterr(parse_url("tcp:/"), "argument")
+nanotest(is.character(random()))
+nanotest(nchar(random(2)) == 4L)
+nanotest(length(random(4L, convert = FALSE)) == 4L)
+nanotesterr(random(1e4L), "54")
 nanotesterr(random("test"), "integer")
+nanotesterr(parse_url("tcp:/"), "argument")
 for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
 nanotest(strcat("hello ", "world") == "hello world")
@@ -375,6 +390,8 @@ nanotest(strcat("hello ", "world") == "hello world")
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
 if (length(s)) {
   nanotestnn(recv(s, block = 500L))
+  nanotest(is.character(opt(s, "ws:response-headers")))
+  nanotesterr(opt(s, "ws:request-headers") <- "test\n", 24)
   nanotest(is.integer(send(s, "message1", block = 500L)))
   nanotestnn(recv(s, block = FALSE))
   nanotest(is.integer(send(s, "message2", block = FALSE)))
@@ -446,6 +463,7 @@ nanotest(!.unresolved(fakectx))
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
 nanotesterr(close(fakectx), "valid Context")
+nanotesterr(reap(fakectx), "valid")
 fakestream <- `class<-`("test", "nanoStream")
 nanotestp(fakestream)
 fakesession <- `class<-`("test", "ncurlSession")
@@ -475,7 +493,7 @@ nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d
 nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
 nanotest(nchar(sha512(c(1.1, 2.2), as.raw(1L))) == 128L)
 nanotest(length(sha512("testing", convert = FALSE)) == 64L)
-nanotest(length(sha384(as.list(random(1e4L)), "KEY", convert = FALSE)) == 48L)
+nanotest(length(sha384(as.list(random(1e3L)), "KEY", convert = FALSE)) == 48L)
 nanotest(nchar(sha384(7000L, key = NULL, convert = TRUE)) == 96L)
 nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 28L)
 nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
